/******************************************************************************
GNBCreditsText.cc

	Copyright © 1997 by John Lindal. All rights reserved.

 ******************************************************************************/

#include "GNBHelpText.h"

const JCharacter* kCreditsName  = "GNBCredits";
const JCharacter* kCreditsTitle = "Credits";

const JCharacter* kCreditsText =

"<html>"
"You can download the latest version of <a href=http://www.cco.caltech.edu/~glenn/notebook/>"
"the Buzzer Electronic Notebook</a> from the <a href=ftp://ftp.cco.caltech.edu/pub/glenn/Notebook/>"
"ftp site</a>."
"<p>"
"It is a huge amount of work to write a program from scratch.  All forms of "
"encouragement (including suggestions, encouragement, and donations) are "
"therefore gratefully accepted."
"<p>"
"My thanks to everyone who has submitted bug reports, suggestions, and/or code. "
"If you find a bug or have a suggestion for improving this program, please "
"let me know."
"<p>"
"Glenn Bach<br>"
"<a href=mailto:glenn@cco.caltech.edu>"
"<address>glenn@its.caltech.edu</address>"
"</a>"
"<p>"
"<hr>"
"<p>"
"This program was written using:"
"<ul>"
"<li> <a href=http://www.cco.caltech.edu/~jafl/jx/>"
"	  JX Application Framework <br></a>"
"     http://www.cco.caltech.edu/~jafl/jx/"
"<p>"
"<li> <a href=http://cs.wustl.edu/~schmidt/ACE.html>"
"	  ACE networking library <br></a>"
"     http://siesta.cs.wustl.edu/~schmidt/ACE.html"
"<p>"
"     ACE is copyrighted by Douglas C. Schmidt and his research group at Washington "
"     University, Copyright (c) 1993-1997, all rights reserved."
"<p>"
"<li> Henry Spencer's Regular Expression library"
"</ul>"
"</html>"
;
