/******************************************************************************
 GNBGettingStartedHelpText.cc

	Copyright © 1998 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "GNBHelpText.h"

const JCharacter* kGettingStartedHelpName  = "GNBGettingStartedHelp";
const JCharacter* kGettingStartedHelpTitle = "Getting started with the Buzzer Electronic Notebook";

const JCharacter* kGettingStartedHelpText =

"<html>"
"<h3>GettingStarted with the Buzzer Electronic Notebook</h3>"

"The Buzzer Electronic Notebook was written to keep track of "
"lots of little pieces of information in one location. It consists of a "
"hierarchy of notes and to-do list items all organized into groups. "

"<p>"

"To get started, simply start adding notes, to-do list items and groups "
"as desired, by using the menu items or the buttons on the button bar. "


;