/******************************************************************************
 GNBCategoryNode.h

	Interface for GNBCategoryNode class.

	Copyright © 1998 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_GNBCategoryNode
#define _H_GNBCategoryNode

#include "GNBBaseNode.h"

class JString;
class istream;
class JTree;

class GNBCategoryNode : public GNBBaseNode
{
public:

	GNBCategoryNode(JNamedTreeNode* parent = NULL,
						const char* name = "BASE");

	GNBCategoryNode(JTree* tree, 
						const JCharacter* name);

	virtual ~GNBCategoryNode();

private:

	// not allowed

	GNBCategoryNode(const GNBCategoryNode& source);
	const GNBCategoryNode& operator=(const GNBCategoryNode& source);

};

#endif
