/******************************************************************************
 GNBNoteEditorHelpText.cc

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "GNBHelpText.h"

const JCharacter* kNoteEditorHelpName  	= "GNBNoteEditorHelp";
const JCharacter* kNoteEditorHelpTitle 	= "Note Editor";

const JCharacter* kNoteEditorHelpText =

"<html>"
"<h3>Note Editor</h3>"

"The editor window lets you edit the contents of a note. You do not need "
"to save it, it is automatically saved periodically, and when you close it. "
"You can export the note to a file by selecting <i>Export note...</i> from "
"the <i>File</i> menu."

;