/******************************************************************************
 GNBGlobals.h

 *****************************************************************************/

#ifndef _H_GNBGlobals
#define _H_GNBGlobals

// we include these for convenience

#include <JString.h>

#include <jXGlobals.h>
#include <jTypes.h>
#include <JXTextClipboard.h>

class GNBApp;
class GNBPrefsMgr;
class GNBTrashDir;
class JXPTPrinter;

GNBApp*				GNBGetApplication();
GNBPrefsMgr*		GNBGetPrefsMgr();
GNBTrashDir*		GNBGetTrashDir();
JXPTPrinter*		GNBGetPTPrinter();

	// called by TestApp

JBoolean 	GNBCreateGlobals(GNBApp* app);
void 		GNBDeleteGlobals();
void		GNBSetTrashDir(GNBTrashDir* dir);

#endif
