/******************************************************************************
 GNBHelpText.h

	Copyright © 1998 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_GNBHelpText
#define _H_GNBHelpText

#include <jTypes.h>

extern JSize kHelpSectionCount;

extern const JCharacter* kHelpSectionName[];
extern const JCharacter* kHelpSectionTitle[];
extern const JCharacter* kHelpSectionText[];

extern const JCharacter* kTOCHelpName;
extern const JCharacter* kTOCHelpTitle;
extern const JCharacter* kTOCHelpText;

extern const JCharacter* kMainHelpName;
extern const JCharacter* kMainHelpTitle;
extern const JCharacter* kMainHelpText;

extern const JCharacter* kNoteEditorHelpName;
extern const JCharacter* kNoteEditorHelpTitle;
extern const JCharacter* kNoteEditorHelpText;

extern const JCharacter* kTrashHelpName;
extern const JCharacter* kTrashHelpTitle;
extern const JCharacter* kTrashHelpText;

extern const JCharacter* kGettingStartedHelpName;
extern const JCharacter* kGettingStartedHelpTitle;
extern const JCharacter* kGettingStartedHelpText;

extern const JCharacter* kChangeLogName;
extern const JCharacter* kChangeLogTitle;
extern const JCharacter* kChangeLogText;

extern const JCharacter* kCreditsName;
extern const JCharacter* kCreditsTitle;
extern const JCharacter* kCreditsText;

#endif