/******************************************************************************
 GNBTOCHelpText.cc

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "GNBHelpText.h"

const JCharacter* kTOCHelpName  = "GNBTOCHelp";
const JCharacter* kTOCHelpTitle = "Table of Contents";

const JCharacter* kTOCHelpText =

"<html>"
"<h3>Table of Contents</h3>"

"<ul>"
"<li> <a href=jxhelp:GNBGettingStartedHelp>Getting started</a>"
"<p>"
"<li> <a href=jxhelp:GNBMainHelp>Main window.</a>"
"<p>"
"<ul>"
"<li> <a href=jxhelp:GNBMainHelp#Notes>Notes</a>"
"<li> <a href=jxhelp:GNBMainHelp#ToDo>To do list items</a>"
"</ul>"
"<p>"
"<li> <a href=jxhelp:GNBNoteEditorHelp>Note editor</a>"
"<p>"
"<li> <a href=jxhelp:GNBTrashHelp>Discards window</a>"
"<p>"
"<li> <a href=jxhelp:GNBChanges>Changes</a>"
"<p>"
"<li> <a href=jxhelp:GNBCredits>Credits</a>"
"<p>"
"</ul>"
"For more information about a particular feature, simply click on the underlined text."

"</html>"
;