/******************************************************************************
 GNBHelpPrefObject.h

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_GNBHelpPrefObject
#define _H_GNBHelpPrefObject

#include <JPrefObject.h>	

class GNBHelpPrefObject : public JPrefObject
{
public:

	GNBHelpPrefObject(JPrefsManager* prefsMgr, const JPrefID& id);

	virtual ~GNBHelpPrefObject();

protected:

	virtual void	ReadPrefs(istream& input);
	virtual void	WritePrefs(ostream& output) const;


private:

	// not allowed

	GNBHelpPrefObject(const GNBHelpPrefObject& source);
	const GNBHelpPrefObject& operator=(const GNBHelpPrefObject& source);
};

#endif
