/******************************************************************************
 GNBChangeLogText.cc

	Copyright © 1999 by Glenn Bach. All rights reserved.

 ******************************************************************************/

#include "GNBHelpText.h"

const JCharacter* kChangeLogName  = "GNBChanges";
const JCharacter* kChangeLogTitle = "Changes from previous versions";

const JCharacter* kChangeLogText =

"Please send all bug reports and suggestions for features to Glenn Bach."
"<p>"
"glenn@its.caltech.edu"
"<p>"
"----------"
"<p>"
"<b>1.0.1</b>"
"<p>"

"Added menu commands for toggling the to-do items."

"<p>"
"<p>"
"<b>1.0</b>"
"<p>"

"Added <i>Export selected notes...</i> and <i>Print selected notes...</i> to "
"export or print all of the selected notes."

"<p>"

"There were just a few other minor adjustments in behavior and appearance in "
"this release."

"<p>"
"<b>0.5</b>"
"<p>"
"initial release"
;
