/******************************************************************************
 GAddressBaseTreeNode.h

	Interface for GAddressBaseTreeNode class.

	Copyright © 1998 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#ifndef _H_GAddressBaseTreeNode
#define _H_GAddressBaseTreeNode

#include "JNamedTreeNode.h"

class JString;
class ostream;

class GAddressBaseTreeNode : public JNamedTreeNode
{
public:

	GAddressBaseTreeNode(JTree*	tree,
						JNamedTreeNode* parent = NULL,
						const char* name = "BASE",
						const JBoolean isBranch = kTrue);

	virtual ~GAddressBaseTreeNode();

	virtual JString	GetFullPathAndName() const = 0;
	virtual void	Save(ostream& os);

private:

	// not allowed

	GAddressBaseTreeNode(const GAddressBaseTreeNode& source);
	const GAddressBaseTreeNode& operator=(const GAddressBaseTreeNode& source);

};

#endif
