/******************************************************************************
 GNBTrashHelpText.cc

	Copyright © 1999 by Glenn W. Bach. All rights reserved.

 ******************************************************************************/

#include "GNBHelpText.h"

const JCharacter* kTrashHelpName  	= "GNBTrashHelp";
const JCharacter* kTrashHelpTitle 	= "Discard Window";

const JCharacter* kTrashHelpText =

"<html>"
"<h3>Discards Window</h3>"

"The discards window contains all of the items that you have deleted from "
"the main window. To undelete an item, simply drag it back into the main window. "
"The discarded items can be flushed from the discards window by selecting "
"<i>Flush discards</i> from the <i>Discards</i> menu."

"<p>"

"All of the items in the discard window will be deleted if you quit the program."
;